/*
 * Universitat Oberta de Catalunya
 * Ingeniería de Informática, 2º ciclo
 * Proyecto fin de carrera- Diseño e implementación de un framework de presentación
 * 
 * Autor:Enrique Mengíbar Vázquez
 * Fecha: Enero 2012 
 * 
 * Description: La presente clase simula una BD sin persistencia de ningún tipo. Los valores iniciales se cargan 
 * a partir de un fichero de propiedades. La BD está compuesta de una tabla de noticias y otra de comentarios, 
 * la relación entre noticias y comentarios es uno a muchos
 * 
 */

package org.sofp.test.basededatos;

import java.io.InputStream;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

public class Basededatos {
	
	static final String PREFIJO_NOTICIA = "not";	// Prefijo para identificar una línea que contiene una noticia
	static final String PREFIJO_COMENTARIO = "com";	// Prefijo para identificar una línea que contiene una noticia
	static final String DELIMITADOR = "#";			// Caracter que usamos como delimitador para separar campos enun registro

	/* Usamos una colección de tipo Map que actúa como un Hashmap para almacenar una colección de noticias
	 * Una BD se compone únicamente de una lista de noticias. Cada noticia tiene una clave primaria que se genera
	 * aleatoriamente durante el alta de la noticia.
	 */
	
	Map<String, Noticia> noticias;

	/* En el consstructor de la clase, leemos de un fichero de parámatros y cargamos los valores, para poder trabajar con un
	 * conjunto de valores de partida
	 */
	public Basededatos() {

		// Inicializamos el Mapa de las noticias, la clave es un String y el objeto almacenado es de la clase Noticia
		this.noticias = new HashMap<String, Noticia>();

		/* Vamos a trabajar con dos ficheros de propiedades, uno para los comentarios y otro para las noticias
		/* el fichero tiene el formato siguiente: clave=campo1_campo2_campo3_campo4
		 * donde usamos un delimitador para separar los campos y una línea por registro, el campo clave no se usa
		 * sólo lo usaremos para vincular un comentario a una noticia
		 */
		Properties propiedadesNoticias = new Properties();
		Properties propiedadesComentarios = new Properties();

		// Cargamos los ficheros de propiedades, si se produce algún tipo de error capturamos la excepción y 
		// escribimos un mensaje en el log del servidor de aplicaciones
		try {

			/* Cargamos ambos ficheros de propiedades, obtenemos el contexto del classloader del thread actual
			 * cargamos el mismo fichero dos veces en distintos ImputStream, uno lo recorreremos buscando las noticias
			 * y otro buscando los comentarios de cada noticia
			 */
			InputStream streamNoticias = Thread.currentThread().getContextClassLoader().getResourceAsStream("basededatos.properties");
			InputStream streamComentarios = Thread.currentThread().getContextClassLoader().getResourceAsStream("basededatos.properties");

			// Cargamos los stream que hemos definico
			propiedadesNoticias.load(streamNoticias);
			propiedadesComentarios.load(streamComentarios);

		} catch (Exception e) {
			// Si se produce algún error en la carga de algunos de los ficheros mostramos un error
			System.out.println("Ha ocurrido una excepción al abrir el fichero, no se encuentra o está protegido");
		}

		/* Recorremos secuencialmente las noticias, al tratarse de un Map iteramos sobre las claves, recorriéndolas
		 * como si se tratara de una Enumaration. Inicialmente recorremos sólo las keys, para luego utilizar
		 * esta para acceder al objeto almacenado en el hashmap
		 */
		for (Enumeration<Object> eNoticia = propiedadesNoticias.keys(); eNoticia.hasMoreElements();) {
			String claveNoticia = (String) eNoticia.nextElement();				// Obtenemos la clave de la noticia
			String registroNoticia = propiedadesNoticias.getProperty(claveNoticia);		// Obtenemos la línea completa de la noticia por su clave 

			/* Si la clave comienza por el prefijo "not" es una noticia, en caso contrario la ignoramos
			 * se trata de un comentario que trataremos luego
			 */
			if (claveNoticia.startsWith(PREFIJO_NOTICIA)) {

				// Usando el caracter delimitador, desglosamos cada campo en un arry
				String[] temp = registroNoticia.split(DELIMITADOR);
				
				// Asignamos los valores correspondientes sa cada campo, según la posición que ocupan en el registro
				String autor = temp[0];
				String titular = temp[1];
				String descripcion = temp[2];
				String enlace = temp[3];
				String email = temp[4];

				// Añadimos la noticia al Map, creamos la noticia directamente con su constructor al llamar al método addNoticia
				String idNoticia = this.addNoticia(new Noticia( autor,  email,  titular,  descripcion,  enlace));
				

				/* Por cada noticia que leemos, aquellas cuyo prefijo en el fichero de propiedades es el definido
				 * volvemos a recorrer el fichero de propiedades que tenemos en un Properties distinto (pero apuntando al mismo fichero)
				 * ahora buscaremos los registros que corresponden a comentarios de la noticia en curso
				 */
				for (Enumeration eComentario = propiedadesComentarios.keys(); eComentario.hasMoreElements();) {

					// Obtenemos la clave del comentario
					String claveComentario = (String) eComentario.nextElement();			
					
					// Comprobamos si se trata de un comentario
					if (claveComentario.startsWith(PREFIJO_COMENTARIO)) { 
						
						// Obtenemos el registro que corresponde al comentario
						String registroComentario = propiedadesComentarios.getProperty(claveComentario);	

						/* A partir de la clave en el fichero de propiedades de la noticia, dividiendo usando el delimitador
 						 * obtenemos su id, este id sólo es válido en el fichero properties para vincular noticias con comentarios
						 * en la simulación de tabla que hacemos, la clave es un valor UUID aleatorio
						 */
						
						// Dividimos la clave de la noticia usando el caracter deliminator
						String[] claveNoticiaArray = claveNoticia.split(DELIMITADOR);
						// Obtenemos el id de la noticia de la segunda posición del array (NOT_XX)
						String nNoticia = claveNoticiaArray[1];

						// Dividimos la clave del comentario usando el caracter delimitador
						String[] idComentarioArray = claveComentario.split(DELIMITADOR);
						// Obtenemos el ide del comentario de la segunda posición del array (COM_XX_XX)
						String nComentario = idComentarioArray[1];

						/* Comprobamos si el identificador de la noticia y el del comentario son iguales
						 * en cuyo caso se trata de un comentarioque se corresponde con la noticia
						 */
						if (nNoticia.compareTo(nComentario) == 0) {
							
							// Dividimos el registro del Comentario en partes usando el caracter delimitador
							String[] comentarioArray = registroComentario.split(DELIMITADOR);							
							String autorComentario = comentarioArray[0];	// Obtenemos el autor del primer campo
							String comentario = comentarioArray[1];			// Obtenemos el texto del comentario del segundo campo
							
							/* Añadimos el comentario a la noticia, hemos guardado
							 * en la variable idNoticia una referencia a la noticia en curso
							 * usamos el método addComentario, con una referencia al id de la noticia como
							 * parámetro junto con el propio objeto de la clase comentario que instanciamos 
							 * en la propia llamada
							 */							
							this.addComentario(idNoticia, new Comentario(autorComentario, comentario));
						}
					}
				}
			}
		}
	}

	// Añadimos una noticia, devolvemos el id de la noticia, recordemos que es un 
	// UUID aleatorio que se asigna en el constructor de la propia noticia
	public String addNoticia(Noticia n) {
		String id = n.getId();		// Obtenemos el id de la noticia a añadir	
		this.noticias.put(id, n);	// Lo guardamos en el Map y usamos el id de la noticia como Key del Map
		return (id);				// Devolvemos el id de la noticia añadida
	}

	// Borra una noticia, se pasa como argumento la propia noticia
	// Se localiza a partir del Key del Map, que recordemos, coincide con el id de la Noticia
	public void deleteNoticia(Noticia n) {
		this.noticias.remove(n.getId());
	}

	// Borra una noticia, se pasa como argumento un id a la propia noticia
	// Primero se localiza la noticia por su id mediante el correspondiente método
	// get y posteriormente se accede en el Map por su Key al elemento y se elimina
	public void deleteNoticia(String id) {
		Noticia n = getNoticia(id);
		this.noticias.remove(n.getId());
	}
	
	// Obtiene una noticia a partir de su id
	public Noticia getNoticia(String id) {
		return (this.noticias.get(id));
	}

	// Obtiene una colleción simple con todos los valores de las noticias
	// Este método lo usamos para devolver una estructura de datos simple sobre
	// La que podemos iterar fácilmente en las páginas jsp
	public Collection<Noticia> getNoticias() {
		return this.noticias.values();
	}
	
	// Añade un comentario a una noticia, localizamos la noticia por su campo id
	public void addComentario(String idNoticia, Comentario comentario) {
		Noticia noticia = this.noticias.get(idNoticia);
		noticia.addComentario(comentario);
	}

	// Obtenemos una colección con los comentarios de una noticia, accedemos a la noticia por su campo id
	public Collection<Comentario> getComentarios(String idNoticia) {
		Noticia noticia = this.noticias.get(idNoticia);
		return (noticia.getComentariosCollection());
	}
	
	// Metodo toString para depuración, recorre las noticias y las devuelve como cadena, por 
	// cada noticia a su vez, obtiene todos los comentarios asociados
	public String toString() {

		StringBuilder builder = new StringBuilder();

		// Obtendo un iterator de la colección
		for (Iterator i = noticias.entrySet().iterator(); i.hasNext();) {			
			Map.Entry elemento = (Map.Entry) i.next();		// Obtendo el elemento en curso
			String clave = (String) elemento.getKey();		// Obtengo la clave del elemento
			Noticia valor = (Noticia) elemento.getValue();	// Obtengo el valor a partir de la clave
			builder.append("clave=" + clave + " " + valor);	// Añado un mensaje con la clave y la Noticia
			builder.append("\r\n");							// Línea en blanco para facilitar la lectura en el log
		}
		
		return builder.toString();
	}
}
