/**
 * Universitat Oberta de Catalunya
 * Ingeniería de Informática, 2º ciclo
 * Proyecto fin de carrera- Diseño e implementación de un framework de presentación
 * 
 * @author      Enrique Mengíbar Vázquez 
 * @version     1.0    
 * 
 */

package org.sofp.ambito;

import java.io.IOException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.PageContext;

/**
 * Contiene métodos para trabajar con los ámbitos, permite definir y recuperar 
 * variables de una forma homogenea, independientemente del ámbito sobre el que se definan (sesion, 
 * aplicación, request, page). Trabaja con el contexto del servlet y con los objetes request y response
 * 
 * @author      Enrique Mengíbar Vázquez 
 * @version     1.0   
 *  
 */
public final class Contexto {

	// HttpServletRequest y HttpServletResponse quienes contienen la información de la página que invocó al servlet
	// commandBroker funciona como broker del propio HttpServlet
	private HttpServlet commandBroker;		
	private HttpServletRequest request;		
	private HttpServletResponse response;

	 /**
     * Crea un objeto Contexto a partir de los datos de la petición: request, response y una referencia al propio servlet
     * @param commandBroker Referencia al applet 
     * @param request Referencia al objeto request de la petición
     * @param response Referencia al objeto response de la petición
     */
	public Contexto(HttpServlet commandBroker, HttpServletRequest request,
			HttpServletResponse response) {
		this.commandBroker = commandBroker;
		this.request = request;
		this.response = response;
	}

	/** 
	 * Obtenemos una referencia al atributo commandBroker que contiene la referencia al servlet 
	 * @return referencia al contexto del servlet
	 */
	public HttpServlet getCommandBroker() {
		return this.commandBroker;
	}

	/**
	 * Obtenemos una referencia al objeto request de la petición
	 * @return objeto request de la petición
	 */
	public HttpServletRequest getRequest() {
		return this.request;
	}

	/**
	 * Obtenemos una referencia al objeto response de la petición
	 * @return objeto response de la petición
	 */
	public HttpServletResponse getResponse() {
		return this.response;
	}

	/**
	 * Obtenemos la sesión 
	 * @return objeto con la sesion
	 */
	public HttpSession getSession() {
		return getRequest().getSession();
	}

	/**
	 * Obtenemos el contexto del servlet
	 * @return contexto del servlet
	 */
	public ServletContext getServletContext() {
		return this.commandBroker.getServletContext();
	}
	
	/**
	 * Establece el valor de una variable de entorno
	 * @param nombre nombre de la variable a establecer
	 * @param valor valor de la variable
	 * @param ambito ambito de la variable, puede ser requet, aplicacion y sesión
	 * @throws ServletException si el ambito no es correcto se lanza la excepción correspondiente
	 * @throws NullPointerException si el nombre de la variable es nulo se lanza la excepción correspondiente
	 */
	public void setVariableAmbito(Object nombre, Object valor, int ambito)
			throws ServletException, NullPointerException {
		
		// si el valor del nombre es nulo lanzamos una excepción
		if (nombre == null)
			throw new NullPointerException("Ambito nulo");
		
		// Comprobamos el valor del ámbito y establecemos la variable obteniendo una referencia al
		// objeto request, session o al contexto del servlet
		switch (ambito) {
		// ámbito tipo request
		case PageContext.REQUEST_SCOPE:
			getRequest().setAttribute(nombre.toString(), valor);
			break;
		// ámbito de sesión 
		case PageContext.SESSION_SCOPE:
			getSession().setAttribute(nombre.toString(), valor);
			break;
		// ámbito de página
		case PageContext.APPLICATION_SCOPE:
			getServletContext().setAttribute(nombre.toString(), valor);
			break;
		// si el ámbito no es correcto lanzamos una excepción
		default:
			throw new ServletException("El valor del parámetro ámbito " + ambito + " no es válido");
		}
	}

	/**
	 * Recupera el valor de una variable de entorno
	 * @param nombre nombre de la variable a establecer
	 * @param ambito ambito de la variable, puede ser requet, aplicacion y sesión
	 * @throws ServletException si el ambito no es correcto se lanza la excepción correspondiente
	 * @throws NullPointerException si el nombre de la variable es nulo se lanza la excepción correspondiente
	 */	
	public Object getVariableAmbito(Object nombre, int ambito)
			throws ServletException {
		// 
		if (nombre == null)
			throw new NullPointerException("El nombre de la variable es nulo");
		
		// Comprobamos el valor del ámbito y obtenemos la variable obteniendo una referencia al
		// objeto request, session o al contexto del servlet
		switch (ambito) {
		// ámbito request
		case PageContext.REQUEST_SCOPE:
			return getRequest().getAttribute(nombre.toString());
		// ámbito sesión
		case PageContext.SESSION_SCOPE:
			return getSession().getAttribute(nombre.toString());
		// ámbito aplicación
		case PageContext.APPLICATION_SCOPE:
			return getServletContext().getAttribute(nombre.toString());
			// si el ámbito no es correcto lanzamos una excepción
		default:
			throw new ServletException("El valor del parámetro ámbito " + ambito + " no es válido");
		}
	}

	/**
	 * Hacemos una redirección a la vista a partir de una referencia al request de la petición
	 * obtenemos el correspondiente dispatcher
	 * @param path página a la que queremos redireccionar
	 * @throws IOException Si la vista no existe se lanza una excepción
	 * @throws ServletException Si no se puede obtener el dispatcher del request se lanza una excepción
	 * @throws IllegalStateException En caso de que request o response tengan referencias erroneas se lanza una excepción
	 */
	public void forward(String path) throws IOException, ServletException,
	IllegalStateException {
		RequestDispatcher dispatcher = getRequest().getRequestDispatcher(path);
		if (dispatcher == null)
			throw new ServletException("No se puedo obtener el a dispatcher para el path " + path);
		dispatcher.forward(this.request, this.response);
	}
}
