/**
 * Universitat Oberta de Catalunya
 * Ingeniería de Informática, 2º ciclo
 * Proyecto fin de carrera- Diseño e implementación de un framework de presentación
 * 
 * @author      Enrique Mengíbar Vázquez 
 * @version     1.0    
 */

package org.sofp.config;

import java.util.HashMap;
import java.util.Map;

import org.sofp.ambito.Contexto;
import org.sofp.excepciones.CommandNoEncontradoException;
import org.sofp.excepciones.FormularioNoEncontradoExcepcion;
import org.sofp.excepciones.VistaNoEncontradaException;
import org.sofp.forms.FormBean;

/**
 * Contiene métodos para trabajar con el elemento comando
 * de la configuración del framework
 *  
 * @author      Enrique Mengíbar Vázquez  
 *
 */
public class ConfigComando {

	// Path que identifica al comando en la url
	private String path;

	// Clase donde se implementa el comando
	private String clase;

	// Nombre del formulario asociado al comando
	private String formulario;

	// Mapa con las vistas definicidas para el comando
	private Map<String, ConfigVista> mapVistas;

	// Referencia a la propia configuración global
	private ConfiguracionGlobal configuracionGlobal;

	/**
	 * Devuelve la configuración global
	 * @return Devuelve la configuración global
	 */
	public ConfiguracionGlobal getConfiguracionGlobal() {
		return configuracionGlobal;
	}

	/**
	 * Establece el valor de la configuración global
	 * @param configuracionGlobal Valor de la configuración global
	 */
	public void setConfiguracionGlobal(ConfiguracionGlobal configuracionGlobal) {
		this.configuracionGlobal = configuracionGlobal;
	}

	/**
	 * Constructor del comando, crea el mapa de vista
	 */
	public ConfigComando() {
		// inicializamos el mapa de las vistas como una tabla hash
		mapVistas = new HashMap<String, ConfigVista>();
	}

	/**
	 * Obtenemos el valor correspondiente de la clase de un comando
	 * @return Clase que se corresponde a un comando
	 */
	public String getClase() {
		return clase;
	}

	/**
	 * Establecemos el valor correspondiente a la clase de un comando
	 * @param clase correspondiente a un comando
	 */
	public void setClase(String clase) {
		this.clase = clase;
	}

	/**
	 * Obtenemos el valor del path asociado a un comando
	 * @return Path asociado a un comando
	 */
	public String getPath() {
		return path;
	}

	/**
	 * Establecemos el valor del path de un comando
	 * @param path Path asociado al comando
	 */
	public void setPath(String path) {
		this.path = path;
	}

	/**
	 * Obtenemos el valor del formulario asociado a un comando
	 * @return Valor del formulario asociado a un comando
	 */
	public String getFormulario() {
		return formulario;
	}

	/** Establecemos el valor del formulario asociado a un comando
	 * @param formulario valor del formulario asociado al comando
	 */
	public void setFormulario(String formulario) {
		this.formulario = formulario;
	}

	/**
	 *  Establecemos el mapa de las vistas del comando 
	 * @param mapVistas Mapa de las vistas asociadas al comando
	 */
	public void setMapVistas(Map<String, ConfigVista> mapVistas) {
		this.mapVistas = mapVistas;
	}

	/**
	 * Obtenemos una vista a partir de su nombre
	 * @param nombre Nombre de la vista a obtener
	 * @return Objeto vista
	 * @throws VistaNoEncontradaException
	 */
	public ConfigVista obtenerVista(String nombre)
			throws VistaNoEncontradaException {

		// Obtenemos la vista a partir de su nombre
		ConfigVista vista = getMapVistas().get(nombre);

		// Si la vista no existe
		if (vista == null) {

			// Obtengo una referencia a la configuración global
			ConfiguracionGlobal cg = getConfiguracionGlobal();

			if (cg != null) {
				// Intento localizar la vista como una vista global
				vista = cg.getMapVistasGlobales().get(nombre);
				
				// Si tampoco existe, genero una excepción que será propagada
				if (vista == null)
					throw new VistaNoEncontradaException(nombre);
			}
		}

		// Devuelvo la vista si se pudo localizar
		return vista;
	}

	/**
	 * Añade una vista al comando
	 * @param vista Visque que va a ser asociada a un comando
	 */
	public void addVista(ConfigVista vista) {
		// Añadimos la vista al comando, usamos el nombre de la vista como clave para el hashmap
		if (mapVistas != null)
			mapVistas.put(vista.getNombre(), vista);
	}

	/**
	 * Obtenemos el mapa de vistas asociado a un comando
	 * @return mapa de vistas asociado a un comando
	 */
	public Map<String, ConfigVista> getMapVistas() {
		return (this.mapVistas);
	}


	/**
	 * Localiza en la configuración la clase de tipo formBean que hemos asociado al comando que se va a ejecutar. 
	 * crea dinamicamente una instancia de la clase asociada al formulario
	 * @param commandContext Contexto del servlet
	 * @param listaForm Estructura Hashmap que contiene los formularios definidos en la configuración
	 * @return Devuelve una instancia del formbean poblado con los valores que se han pasado desde la vista
	 * @throws FormularioNoEncontradoExcepcion Si el formulario no existe se propaga la excepción correspondiente
	 * @throws CommandNoEncontradoException Si el comando no existe se propaga la excepción correspondiente
	 */
	public FormBean instanciaFormBean(Contexto commandContext,
			Map<String, ConfigFormulario> listaForm)
					throws FormularioNoEncontradoExcepcion,
					CommandNoEncontradoException {

		// Objeto de la clase FormBean que contendrá los datos de nuestroformulario
		FormBean formBeanPoblado = null;

		// Obtenemos el nombre del formulario asociado al comando
		String nombreFormulario = this.getFormulario();

		String nombreFormBean = null;

		// Si no existe formulario devolvemos un formBean vacio
		if (nombreFormulario == null)
			throw new FormularioNoEncontradoExcepcion(nombreFormulario);
		else {

			// A partir del nombre del formulario obtenemos de la configuración el objeto formulario
			ConfigFormulario formulario = listaForm.get(nombreFormulario);

			// Si no existe el formulario retornamos devolviendo un nulo
			if (formulario == null)
				throw new FormularioNoEncontradoExcepcion(nombreFormulario);
			else {
				try {

					nombreFormBean = formulario.getClase();

					// A continuación creamos una clase de forma dinámica usando
					// técnicas de reflexion. La clase a crear es la asociada a nuestro
					// formulario. La reflexión en Java es la técnica que nos permite
					// modificar la estructura del código de manera dinámica (en tiempo de ejecución)
					Class formClass = Class.forName(nombreFormBean);

					// Determinamos si la clase representada en formClass es la
					// misma o una superclase de la clase FormBean. Todos las clases 
					// que estamos usando para el mapeo de los formularios
					// extienden de la clase formBean del paquete org.soft.forms
					if (FormBean.class.isAssignableFrom(formClass)) {
						// Creamos una instancia de la clase asociada a nuestro formulario
						formBeanPoblado = (FormBean) formClass.newInstance();
					}

				} catch (Exception e) {
					// Si el comando no se encuentra, lanzamos la excepción correspondiente
					throw new CommandNoEncontradoException(nombreFormBean);
				}
			}
		}

		// devolvermos el formbean poblado dinámicamente
		return formBeanPoblado;
	}

}