/**
 * Universitat Oberta de Catalunya
 * Ingeniería de Informática, 2º ciclo
 * Proyecto fin de carrera- Diseño e implementación de un framework de presentación
 * 
 * @author      Enrique Mengíbar Vázquez 
 * @version     1.0    
 * 
 */

package org.sofp.config;

import static org.apache.commons.digester3.binder.DigesterLoader.newLoader;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Map.Entry;

import org.apache.commons.digester3.Digester;
import org.apache.commons.digester3.xmlrules.FromXmlRulesModule;
import org.sofp.Constantes;
import org.xml.sax.SAXException;

/**
 * Representa la configuración global del framework
 * Algunos métodos son invocados por el parser Digester durante el parseo del fichero
 * también se dispone de métodos para obtener los valores de configuración
 * 
 * @author Enrique Mengíbar Vázquez 
 *
 */
public class ConfiguracionGlobal {

	// Mapa de comandos
	private Map<String, ConfigComando> mapComandos;

	// Mapa de formularios
	private Map<String, ConfigFormulario> mapFormularios;

	// Mapa de vistas globales
	private Map<String, ConfigVista> mapVistasGlobales;

	// Mapa de locales
	private ConfigLocale mapLocale;

	// Local del navegador
	Locale localeNavegador;

	/**
	 * Constructor
	 * Inicializamos las estructuras Hasmap
	 */
	public ConfiguracionGlobal() {
		mapComandos = new HashMap<String, ConfigComando>();
		mapFormularios = new HashMap<String, ConfigFormulario>();
		mapVistasGlobales = new HashMap<String, ConfigVista>();
	}

	/**
	 * Obtiene el valor del locale del navegador
	 * @return Locale del navegador
	 */
	public Locale getLocaleNavegador() {
		return localeNavegador;
	}

	/**
	 * Establece el valor del locale delnavegador
	 * @param localeNavegador Valor del locale del navegador
	 */
	public void setLocaleNavegador(Locale localeNavegador) {
		this.localeNavegador = localeNavegador;
	}

	/**
	 * Obtiene el mapa de locales de la configuración
	 * @return Mapa de locales de la configuración
	 */
	public ConfigLocale getMapLocale() {
		return mapLocale;
	}

	/**
	 * Establece el valor del mapa de locales de la configuración
	 * @param mapLocale Mapa de locales de la configuración
	 */
	public void setMapLocale(ConfigLocale mapLocale) {
		this.mapLocale = mapLocale;
	}

	/**
	 * Establece el valor del mapa de locales de la configuración
	 * @param mapRecursosMensajes
	 */
	public void setMapRecursosMensajes(ConfigLocale mapRecursosMensajes) {
		this.mapLocale = mapRecursosMensajes;
	}

	/**
	 * Obtiene el mapa de locales de la configuración
	 * @return Mapa de locales de la configuración
	 */
	public Map<String, ConfigVista> getMapVistasGlobales() {
		return mapVistasGlobales;
	}

	/**
	 * Establece el valor del mapa de las vistas globales de la configuración
	 * @param mapVistasGlobales Mapa de las vistas globales de la configuración
	 */
	public void setMapVistasGlobales(Map<String, ConfigVista> mapVistasGlobales) {
		this.mapVistasGlobales = mapVistasGlobales;
	}

	/**
	 * Añade una vista global al a configuración
	 * @param vista Vsta global
	 */
	public void addVistaGlobal(ConfigVista vista) {
		if (mapVistasGlobales != null)
			this.mapVistasGlobales.put(vista.getNombre(), vista);
	}

	/** Añade un comando a la configuración
	 * @param comando Comando a añadir a la configuración
	 */
	public void addComando(ConfigComando comando) {
		if (mapComandos != null) {
			mapComandos.put(comando.getPath(), comando);
			comando.setConfiguracionGlobal(this);
		}
	}

	/**
	 * Añadimos un formulario a la configuración global
	 * @param formulario
	 */
	public void addFormulario(ConfigFormulario formulario) {
		if (mapFormularios != null)
			this.mapFormularios.put(formulario.getNombre(), formulario);
	}

	/**
	 * Establece el valor del locale
	 * @param locale Valor de locale
	 */
	public void setLocale(ConfigLocale locale) {
		mapLocale = locale;
	}

	/**
	 * Obtiene un mensaje definido en la configuración de internacionalización de la aplicación
	 * @param fichero Fichero de mensajes properties
	 * @param clave Clave del mensaje a obtener
	 * @return Mensaje correspondiente a la clave indicada como parámetro
	 */
	public String getMensaje(String fichero, String clave) {
		// Obtenemos el ResourceBundle correspondiente a partir del nombre del
		// fichero que hemos usado como índice en
		// el estructura tipo mapa

		try {
			Map<String, ResourceBundle> mapaRecursos = mapLocale.getMapRecursoMensajesBundle();

			ResourceBundle rb = mapaRecursos.get(fichero);
			return (rb.getString(clave));

		} catch (Exception e) {
			return (Constantes.NULL);
		}
	}

	/**
	 * Obtiene el mapa de comandos de la configuración
	 * @return Mapa de comandos de la configuración
	 */
	public Map<String, ConfigComando> getMapComandos() {
		return mapComandos;
	}

	/**
	 * Establece el mapa de comandos de la configuración
	 * @param mapComandos Mapa de comandos de la configuración
	 */
	public void setMapComandos(Map<String, ConfigComando> mapComandos) {
		this.mapComandos = mapComandos;
	}

	/** 
	 * Obtenemos el mapa de formularios de la configuración
	 * @return Mapa de formualrios de la configuración
	 */
	public Map<String, ConfigFormulario> getMapFormularios() {
		return mapFormularios;
	}

	/** 
	 * Establecemos el mapa de formularios de la configuraciñon
	 * @param mapFormularios Mapa de formularios de la configuración
	 */
	public void setMapFormularios(Map<String, ConfigFormulario> mapFormularios) {
		this.mapFormularios = mapFormularios;
	}

	/**
	 * Obtenemos el mapa de vistas globales de la configuración
	 * @return Mapa de vistas globales de la configuración
	 */
	public Map<String, ConfigVista> getVistasGlobales() {
		return mapVistasGlobales;
	}

	/** 
	 * Establecemos el mapa de vistas globales de la configuración
	 * @param mapVistasGlobales Mapa de vistas globales de la configuración
	 */
	public void setVistasGlobales(Map<String, ConfigVista> mapVistasGlobales) {
		this.mapVistasGlobales = mapVistasGlobales;
	}

	/**
	 * Parse de los valores de configuración, invoca al parser Digester y a partir de las reglas
	 * especificadas en el fichero digesterReglas.xml realiza el parseo y poblado de la clase Configuración Global 
	 * con los datos del fichero sofp-config.xml
	 * @param reglas Fichero de definición de reglas para Digester
	 * @param ficheroXML Fichero de configuración del framework
	 */
	public void parsearValoresConfiguracion(String reglas, String ficheroXML) {

		// Obtenemos la url del fichero de reglas
		final URL xml = getClass().getResource(reglas);

		// Instanciamos un digester
		Digester digester = newLoader(new FromXmlRulesModule() {

			@Override
			// La carga de las reglas se realiza desde la URL que apunta
			// al fichero de definición de reglas y que hemos pasado como
			// parámetro
			protected void loadRules() {
				loadXMLRules(xml);
			}

		}).newDigester();

		// Asociamos al digester el objeto que contiene la configuración. Como
		// se trata de un parseo 'dirigida por eventos', será la invocación a 
		// los métodos apropiados del objeto conf los que irán poblando la 
		// configuración de forma automática.
		digester.push(this);

		// Iniciamos el parseo del fichero de configuración
		try {
			digester.parse(getClass().getResource(ficheroXML));
		} catch (IOException e) {
			e.printStackTrace(); 	// mostramos el mensaje de error en caso de que se produzca una excepción
		} catch (SAXException e) {
			e.printStackTrace();	// mostramos el mensaje de error en caso de que se produzca una excepción
		}
	}

	// cargamos los ficheros de recursos, en caso de que no se haya especificado
	// un locale, cogemos el del browser
	public void cargarFicherosRecursos(Locale localeBrowser, String localeConfiguracion) {

		// Damos un valor al mapa de recursos de mensajes
		Map<String, ConfigRecursoMensajes> crm = this.getMapLocale().getMapRecursoMensajes();

		// Obtenemos el mapa de recursos
		ConfigLocale configLocale = this.getMapLocale();

		// Iteramos sobre el mapa de recursos de mensajes que se han definido en la configuración
		// por cada uno de ellos 
		for (Iterator it = crm.entrySet().iterator(); it.hasNext();) {
			
			Map.Entry item = (Entry) it.next();
			Object clave = item.getKey();
			// Obtenemos el valor del recurso de mensajes actual mientras recorremos la colección
			ConfigRecursoMensajes valor = (ConfigRecursoMensajes) item.getValue();

			// Definimos cual será el local actual de la configuración
			// Si no se ha definido ninguno en el fichero de recursos se tomará el correspondiente al navegador
			Locale locale = null;
			if (localeConfiguracion == null || localeConfiguracion.isEmpty()) {
				// Si no se ha definido en la configuración ninguna referencia al locale 
				// establecemos la del navegador como valor por defecto
				locale = localeBrowser;
			} else {
				// Desglosamos el valor del locale en el código del idioma y el del país
				String[] temp;
				temp = localeConfiguracion.split("-");

				// Valor del idimos
				String idioma = temp[0];
				// Valor del país
				String pais = temp[1];
				
				// Creamos un nuevo locale
				locale = new Locale(idioma, pais);
			}

			if (valor != null && locale != null) {
				// Creamos un recourceBoundle para el locale que se va a usar
				ResourceBundle mensajes = ResourceBundle.getBundle(valor.getPrefijoFichero(), locale);
				// Añadimos a la configuración global el recourceBoundle del
				// mapa de recursos del locale definido como actual
				configLocale.addRecursoBundle(valor.getNombre(), mensajes);
			}
		}
	}
}
